/*
 *-------------------------------------------------------------------
 *
 *   crank.c -- top level crank sensor controller
 *
 * This file has been created by John S. Gwynne for support of 
 * Motorola's 68332 MCU in the efi332 project.
 * 
 * Redistribution and use in source and binary forms are permitted
 * provided that the following conditions are met:
 * 1. Redistribution of source code and documentation must retain
 *    the above authorship, this list of conditions and the
 *    following disclaimer.
 * 2. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * This software is provided "AS IS" without warranty of any kind,
 * either expressed or implied, including, but not limited to, the
 * implied warranties of merchantability, title and fitness for a
 * particular purpose.
 *
 *------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include "system.h"
#include "engineConfig.h"
#include "tpudriver.h"
#include "crank.h"
#include "crankInline.h"
#include "global.h"
#include "logger.h"

int crankState = crankStalled;

rtems_task crankSenor(
  rtems_task_argument argument
  )
{
    rtems_id          tid;
    rtems_status_code status;
    int tpuStat;
    int rpm;
    char s[LogLineLength];

    status = rtems_task_ident( RTEMS_SELF, RTEMS_SEARCH_ALL_NODES, &tid );
    directive_failed( status, "rtems_task_ident" );

    crankStartSensorChannel();

    while( FOREVER ) {

	tpuStat = crankBlockForStatChange();
	
	switch (tpuStat) {
	case crankToothInterrupt:
	    crankState = crankRunning;
	    crankDisableToothInterrupt();
	    LOG(LOG_NOTICE,"crank: leaving stalled state",LogLineLength);
	    crankEnableAllChannels();
	    break;
	case crankSynchronize:
	    crankState = crankTransitioning;
	    crankSetToothInterrupt(StallResetCount);
	    LOG(LOG_INFO,"crank: synchronizing condition detected",
               LogLineLength);
	    break;
	case crankInsufficientTransitions:
	    if (crankState != crankStalled) {
		rpm = ( crankState == crankRunning ? engineRPM() : 0 );
		crankState = crankStalled;
		LOG(LOG_WARNING,"crank: insufficient transitions",
		    LogLineLength);
		crankRestartTPU(rpm);
	    }
	    break;
	case crankGapUnexpectedTransition:
	    if (crankState != crankStalled) {
		rpm = ( crankState == crankRunning ? engineRPM() : 0 );
		crankState = crankStalled;
		LOG(LOG_WARNING,"crank: unexpected transition in the gap",
		    LogLineLength);
		crankRestartTPU(rpm);
	    }
	    break;
	case crankUnexpectedTransition:
	    if (crankState != crankStalled) {
		rpm = ( crankState == crankRunning ? engineRPM() : 0 );
		crankState = crankStalled;
		LOG(LOG_WARNING,"crank: unexpected transition outside the gap",
		    LogLineLength);
		crankRestartTPU(rpm);
	    }
	    break;
	case crankMissedTooth:
	    if (crankState != crankStalled) {
		rpm = ( crankState == crankRunning ? engineRPM() : 0 );
		crankState = crankStalled;
		LOG(LOG_WARNING,"crank: missing tooth detected",LogLineLength);
		crankRestartTPU(rpm);
	    }
	    break;
	case crankExcesiveTimeOut:
	    LOG(LOG_NOTICE,"crank: timeout match exceeds 0x8000",
               LogLineLength);
	    break;
	default:
	    sprintf(s,"crank: unkown tpu status number %i",tpuStat);
	    LOG(LOG_ERROR,s,LogLineLength);
	    break;
	}    
    }
}
