/*
 *-------------------------------------------------------------------
 *
 *   tpuInline.h -- inline functions for tpuInline.c
 *
 * This file has been created by John S. Gwynne for support of 
 * Motorola's 68332 MCU in the efi332 project.
 * 
 * Redistribution and use in source and binary forms are permitted
 * provided that the following conditions are met:
 * 1. Redistribution of source code and documentation must retain
 *    the above authorship, this list of conditions and the
 *    following disclaimer.
 * 2. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * This software is provided "AS IS" without warranty of any kind,
 * either expressed or implied, including, but not limited to, the
 * implied warranties of merchantability, title and fitness for a
 * particular purpose.
 *
 *------------------------------------------------------------------
 */

#ifndef _tpuInline_h_
#define _tpuInline_h_

#include "stdlib.h"
#include "tpu.h"
#include "tpudriver.h"

static void tpuStopAllChannels(void);
__inline__ static void tpuStopAllChannels(void) {
#if TPUincludeSimualtor == 0
    *CPR0 = *CPR1 = 0;
#else
    if (TPUsimulator<8) {
	*CPR1 &= 0x3<<(TPUsimulator*2);
	*CPR0 = 0;
    } else {
	*CPR1 = 0;
	/* abs() just stops a compilation warning */
	*CPR0 &= 0x3<<(abs(TPUsimulator-8)*2);
    }
#endif
}

static void tpuLoadEmulationRAM(void);
__inline__ static void tpuLoadEmulationRAM(void) {
    /* Configure the TRAM */
    *TRAMBAR = ((TRAM_START>>8) & TRAM_ADDR); /* Base address 0xf0000 */
    *TRAMMCR = 0;
    /* upload the micro code */
    bcopy(tpu_buffer,(unsigned char *)TRAM_START,TPU_RAM_SIZE);
}

static void tpuInitTPUMCRandTICR(void);
__inline__ static void tpuInitTPUMCRandTICR(void) {
    *TPUMCR = ( SAM(TPUTCR1,13,TPU_TCR1P) | /* TCR1 Prescaler to 1 */
		SAM(EFI_TPU_IARB,0,TPU_IARB) | /* Interrupt Arbitration */
		TPU_EMU | SAM(TPUPSCK,6,TPU_PSCK));
    *TICR = (SAM(EFI_TPU_CIRL,8,TPU_CIRL) | SAM(EFI_TPU_CIBV,4,TPU_CIBV));
}

static void tpuInterruptCatch(rtems_isr_entry ISR, rtems_vector_number vect);
__inline__ static void tpuInterruptCatch(rtems_isr_entry ISR, 
					 rtems_vector_number vect) {
    rtems_isr_entry dummy;
    rtems_status_code status;

    status = rtems_interrupt_catch(ISR,vect,&dummy);
    directive_failed( status, "rtems_interrupt_catch");
}

static void tpuInstallInterruptVectors(void);
__inline__ static void tpuInstallInterruptVectors(void) {
    int i;
    tpuInterruptCatch(crankISR, TPUcrank+EFI_TPU0);
    for (i=0;i<numberInjectors;i++)
	tpuInterruptCatch(injectorISR,TPUinjector[i]+EFI_TPU0);
    for (i=0;i<numberIgnition;i++)
	tpuInterruptCatch(ignitionISR,TPUignition[i]+EFI_TPU0);
    if (TPUincludeSimualtor)
	tpuInterruptCatch(simulatorISR,TPUsimulator+EFI_TPU0);
}

static void tpuInitFunctionSelectRegisters();
__inline__ static void tpuInitFunctionSelectRegisters() {
    int i, reg[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    
    reg[TPUcrank]=0xf;
    for (i=0;i<numberInjectors;i++)
	reg[TPUinjector[i]]=0xe;
    for (i=0;i<numberIgnition;i++)
	reg[TPUignition[i]]=0xe;
    if (TPUincludeSimualtor)
	reg[TPUsimulator]=0xc;
    
    *CFSR0 = reg[15]<<12 | reg[14]<<8 | reg[13]<<4 | reg[12];
    *CFSR1 = reg[11]<<12 | reg[10]<<8 | reg[9]<<4  | reg[8];
    *CFSR2 = reg[7]<<12  | reg[6]<<8  | reg[5]<<4  | reg[4];
    *CFSR3 = reg[3]<<12  | reg[2]<<8  | reg[1]<<4  | reg[0];
}

static void tpuInitHostSequenceRegisters();
__inline__ static void tpuInitHostSequenceRegisters(){
    int i, reg[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    
    for (i=0;i<numberInjectors;i++)
	reg[TPUinjector[i]]=0x01;
    for (i=0;i<numberIgnition;i++)
	reg[TPUignition[i]]=0x01;
    
    *HSQR0 = reg[15]<<14 | reg[14]<<12 | reg[13]<<10 | reg[12]<<8 |
             reg[11]<<6  | reg[10]<<4  | reg[9]<<2   | reg[8];
    *HSQR1 = reg[7]<<14  | reg[6]<<12  | reg[5]<<10  | reg[4]<<8  |
             reg[3]<<6   | reg[2]<<4   | reg[1]<<2   | reg[0];
}


static void tpuInitChannelInterrputRegister();
__inline__ static void tpuInitChannelInterrputRegister(){
    int i, reg=0;
    
    reg |= 1<<TPUcrank;
    for (i=0;i<numberInjectors;i++)
	reg |= 1<<TPUinjector[i];
    for (i=0;i<numberIgnition;i++)
	reg |= 1<<TPUignition[i];
    if (TPUincludeSimualtor)
	reg |= 1<<TPUsimulator;
    
    *CIER = reg;
}

static void tpuLoadSimPRAM();
__inline__ static void tpuLoadSimPRAM(){
    *(TPU_PR0+TPUsimulator*8+0) = 
       (SAM(0x4,5,0x1e0) | SAM(0x4,2,0x1c) | SAM(0x1,0,0x3));
    *(TPU_PR0+TPUsimulator*8+2) = tpuSimPeriod/2;
    *(TPU_PR0+TPUsimulator*8+3) = tpuSimPeriod;
}

static void tpuLoadCrankPRAM();
__inline__ static void tpuLoadCrankPRAM(){
    *(TPU_PR0+TPUcrank*8+0) = PMMXconfig;           
    *(TPU_PR0+TPUcrank*8+1) = PMMXwaitTeeth;
    *(TPU_PR0+TPUcrank*8+2) = PMMXintTooth;
    *(TPU_PR0+TPUcrank*8+3) = PMMXstartPeriod<<8 | PMMXnumberPeriods;
    *(TPU_PR0+TPUcrank*8+4) = PMMXratio<<8;
    *(TPU_PR0+TPUcrank*8+5) = PMMXlinkStart<<12 | (PMMXlinkCount<<8);
}

static void tpuLoadInjectorPRAM();
__inline__ static void tpuLoadInjectorPRAM(){
    int i;

#warning fix PRAM!!!!!!!!!!!!!!!!!
    for (i=0;i<numberInjectors;i++) {
	*(TPU_PR0+TPUinjector[i]*8+0) = 0;
	*(TPU_PR0+TPUinjector[i]*8+1) = 0;
	*(TPU_PR0+TPUinjector[i]*8+2) = 0;
	*(TPU_PR0+TPUinjector[i]*8+3) = 0;
	*(TPU_PR0+TPUinjector[i]*8+4) = 0;
	*(TPU_PR0+TPUinjector[i]*8+5) = 0;
    }
}

static void tpuLoadIgnitionPRAM();
__inline__ static void tpuLoadIgnitionPRAM(){
    int i;

#warning fix PRAM!!!!!!!!!!!!!!!!!
    for (i=0;i<numberIgnition;i++) {
	*(TPU_PR0+TPUignition[i]*8+0) = 0;
	*(TPU_PR0+TPUignition[i]*8+1) = 0;
	*(TPU_PR0+TPUignition[i]*8+2) = 0;
	*(TPU_PR0+TPUignition[i]*8+3) = 0;
	*(TPU_PR0+TPUignition[i]*8+4) = 0;
	*(TPU_PR0+TPUignition[i]*8+5) = 0;
    }
}

static void tpuStartChannel(int channel, int req, int priority);
__inline__ static void tpuStartChannel(int channel, int req, int priority){
    if (channel<8) {
	*HSRR1 |= req<<(channel*2);
	*CPR1  |= priority<<(channel*2);
	while (*HSRR1 & (req<<(channel*2)));
    } else {
	*HSRR0 |= req<<((channel-8)*2);
	*CPR0  |= priority<<((channel-8)*2);
	while (*HSRR0 & (req<<((channel-8)*2)));
    }
}

static void tpuStartCrank(void);
__inline__ static void tpuStartCrank(void){
    tpuStartChannel(TPUcrank,0x3,crankPriority);
}

static void tpuStartSimulator(void);
__inline__ static void tpuStartSimulator(void){
    tpuStartChannel(TPUsimulator,0x2,simulatorPriority);
}

static void tpuStartInjector(void);
__inline__ static void tpuStartInjector(void){
    int i;
    for (i=0;i<numberInjectors;i++)
	tpuStartChannel(TPUinjector[i],0x3,injectorPriority);
}

static void tpuStartIgnition(void);
__inline__ static void tpuStartIgnition(void){
    int i;
    for (i=0;i<numberIgnition;i++)
	tpuStartChannel(TPUignition[i],0x3,ignitionPriority);
}

static void tpuStopChannel(int channel);
__inline__ static void tpuStopChannel(int channel){
    if (channel<8)
	*CPR1  &= ~(0x3<<(channel*2));
    else
	*CPR0  &= ~(0x3<<((channel-8)*2));
}

static void tpuForceChannelLow(int channel, int priority);
__inline__ static void tpuForceChannelLow(int channel, int priority) {
    *(TPU_PR0+channel*8+0) = 0x13;
    *(TPU_PR0+channel*8+1) = 0x00;
    tpuStartChannel(channel,0x3,priority);
}

static void tpuForceChannelsLow(void);
__inline__ static void tpuForceChannelsLow(void) {
    int i;
    for (i=0;i<numberIgnition;i++)
	tpuForceChannelLow(TPUignition[i],ignitionPriority);
    for (i=0;i<numberInjectors;i++)
	tpuForceChannelLow(TPUinjector[i],injectorPriority);
}

static void tpuForceInjectorsLow(void);
__inline__ static void tpuForceInjectorsLow(void) {
    int i;
    for (i=0;i<numberInjectors;i++)
	tpuForceChannelLow(TPUinjector[i],injectorPriority);
}

static void tpuIntTooth(int count);
__inline__ static void tpuIntTooth(int count) {
    /* not able to really write bytes to PRAM ??? */
    *((unsigned char*)(TPU_PR0)+TPUcrank*16+5)=(unsigned char)count;
    /* zero the status number.... again :( */
    *(TPU_PR0+TPUcrank*8+2) &= 0xff;
}

static void tpuIgnShutDownDelay(int sleepTime);
__inline__ static void tpuIgnShutDownDelay(int sleepTime) {
    rtems_status_code status;
    if ((sleepTime = (sleepTime*TICKS_PER_SECOND) / 1000) == 0) sleepTime=1;
    status = rtems_task_wake_after(sleepTime);
    directive_failed( status, "rtems_task_wake_after" );
}

static void tpuFlushMessageQueue(void);
__inline__ static void tpuFlushMessageQueue(void) {
    rtems_status_code status;
    int count;
    status = rtems_message_queue_flush( Queue_id[TPUQueueID], &count );
    directive_failed( status, "rtems_message_queue_flush" );
}

static void tpuClearInterrupt(int channel);
static void tpuClearInterrupt(int channel) {
        *CISR &= ~(0x01<<channel);
}

static int tpuCrankStat(void);
__inline__ static int tpuCrankStat(void) {
    int i;
    i = (int)( *(volatile unsigned char * const)(TPU_PR0+TPUcrank*8+2));
    *(TPU_PR0+TPUcrank*8+2) &= 0xff;
    return i;
}

static void tpuReadTPUqueue(void *argument);
__inline__ static void tpuReadTPUqueue(void *argument) {
    rtems_status_code status;
    int length;
    status = rtems_message_queue_receive(
       Queue_id[ TPUQueueID ],
       argument,
       &length,
       RTEMS_DEFAULT_OPTIONS,
       RTEMS_NO_TIMEOUT
       );
    directive_failed( status, "rtems_message_queue_receive" );
}


#endif /* _tpuInline_h_ */
