/*
 *-------------------------------------------------------------------
 *
 *   tpudriver.c -- low level driver for the tpu
 *
 * This file has been created by John S. Gwynne for support of 
 * Motorola's 68332 MCU in the efi332 project.
 * 
 * Redistribution and use in source and binary forms are permitted
 * provided that the following conditions are met:
 * 1. Redistribution of source code and documentation must retain
 *    the above authorship, this list of conditions and the
 *    following disclaimer.
 * 2. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * This software is provided "AS IS" without warranty of any kind,
 * either expressed or implied, including, but not limited to, the
 * implied warranties of merchantability, title and fitness for a
 * particular purpose.
 *
 *------------------------------------------------------------------
 */

#include "system.h"
#include "logger.h"
#include "engineConfig.h"
#include "tpu.h"
#include "tpudriver.h"

const unsigned char TPUinjector[] = { TPUinjFireOrder };
const unsigned char TPUignition[] = { TPUignFireOrder };
unsigned int tpuSimPeriod = 0x4000;

/* include the tpu micro code to be up loaded to the tpu RAM */
const unsigned char tpu_buffer[TPU_RAM_SIZE] = {
#include "wheel.tpu"
};

#include "tpuInline.h"

/* This routine should only be called once. The MCR register can only
   be set once after reset */
rtems_device_driver tpu_initialize( 
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument
  ) {
    tpuLoadEmulationRAM();
    tpuInitTPUMCRandTICR();
    tpuInstallInterruptVectors();
    tpuInitFunctionSelectRegisters();
    tpuInitHostSequenceRegisters();
    tpuInitChannelInterrputRegister();
    return 0;
}

/* This function starts the specified tpu operation */
rtems_device_driver tpu_open( 
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument
  ) {
    int retVal=0;
    switch (minor) {
    case DevMinorCrank:
	tpuLoadCrankPRAM();
	tpuStartCrank();
	LOG(LOG_DEBUG,"tpudriver: started crank channel",50);
	break;
    case DevMinorInjector:
	tpuLoadInjectorPRAM();
	tpuStartInjector();
	LOG(LOG_DEBUG,"tpudriver: started injector channel(s)",50);
	break;
    case DevMinorIgnition:
	tpuLoadIgnitionPRAM();
	tpuStartIgnition();
	LOG(LOG_DEBUG,"tpudriver: started ignition channel(s)",50);
	break;
    case DevMinorSimulator:
	tpuLoadSimPRAM();
	tpuStartSimulator();
	LOG(LOG_DEBUG,"tpudriver: started simulator channel",50);
	break;
    default:
	retVal = 1;
	break;
    }
    return retVal;
}

/*
   The argument, in milliseconds, determines the
   time delay between shutting down the injectors and discharging
   the coils. The goal here is to pump out any air-fuel mixture
   before setting off an asynchronous spark. This is more or less
   an emergency shut down routine.
 */
rtems_device_driver tpu_close( 
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument
  ) {
    tpuStopChannel(TPUcrank);
    tpuFlushMessageQueue();
    tpuForceInjectorsLow();
    tpuIgnShutDownDelay(*(int*)argument);
    tpuForceChannelsLow();
    tpuStopAllChannels();
    tpuFlushMessageQueue();
    LOG(LOG_DEBUG,"tpudriver: stopped all tpu channels",50);
    return 0;
}

rtems_device_driver tpu_control( 
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument
  ) {
    int retVal=0;
 
    switch(*(int*)argument) {
    case TPUstatusNumber:
	tpuReadTPUqueue(argument);
	break;
    case TPUSetToothInterrupt:
	tpuIntTooth(*((int*)argument+1));
	break;
    default:
	retVal=1;
	break;
    }
    return retVal;
}


rtems_device_driver tpu_read( 
  rtems_device_major_number  major,
  rtems_device_minor_number  minor,
  void                      *argument
  ) {
    int retVal=0;
 
    switch(*(int*)argument) {
    case TPUengineRPM:
	*(int*)argument = PMMXlp2RPM( PMMXlongPeriod );
	break;
    case TPUengineSinglePeriodRPM:
	*(int*)argument = PMMXsp2RPM( PMMXsingleToothPeriod);
	break;
    default:
	retVal=1;
	break;
    }
    return retVal;
}


rtems_isr crankISR(rtems_vector_number vector) {
    rtems_status_code status;
    int code;
    code = tpuCrankStat();
    status = rtems_message_queue_send( Queue_id[TPUQueueID],&code,sizeof(int));
    /* Interrupts can come faster than we can handle */
/*     directive_failed( status, "rtems_message_queue_send" ); */
    tpuClearInterrupt(TPUcrank);
}

rtems_isr ignitionISR(rtems_vector_number vector) {
#warning missing code!!!!!!!!!!!!!!!!
    tpuClearInterrupt(vector-EFI_TPU0);
}

rtems_isr injectorISR(rtems_vector_number vector) {
#warning missing code!!!!!!!!!!!!!!!!
    tpuClearInterrupt(vector-EFI_TPU0);
}

rtems_isr simulatorISR(rtems_vector_number vector) {
    static int tooth=0;

    if (++tooth==60) tooth=0;
    if (tooth==58 || tooth==59)
	*(unsigned int*)(TPU_PR0+TPUsimulator*8+2) =
	    tpuSimPeriod<<16 | tpuSimPeriod;
    else
	*(unsigned int*)(TPU_PR0+TPUsimulator*8+2) =
	    (tpuSimPeriod/2)<<16 | tpuSimPeriod;
    tpuClearInterrupt(TPUsimulator);
}
